@@ -5,9 +5,9 @@ use std::path::PathBuf;
 
 use anyhow::Context as _;
 use cargo_edit::{
-    find, get_compatible_dependency, get_latest_dependency, registry_url, set_dep_version,
-    shell_note, shell_status, shell_warn, shell_write_stdout, update_registry_index, CargoResult,
-    CrateSpec, Dependency, LocalManifest, RustVersion, Source,
+    get_compatible_dependency, get_latest_dependency, registry_url, set_dep_version, shell_note,
+    shell_status, shell_warn, shell_write_stdout, CargoResult, CertsSource, CrateSpec, Dependency,
+    IndexCache, LocalManifest, RustVersion, Source,
 };
 use clap::Args;
 use indexmap::IndexMap;
@@ -34,10 +34,6 @@ pub struct UpgradeArgs {
     #[arg(long)]
     ignore_rust_version: bool,
 
-    /// Run without accessing the network
-    #[arg(long)]
-    offline: bool,
-
     /// Require `Cargo.toml` to be up to date
     #[arg(long)]
     locked: bool,
@@ -160,12 +156,10 @@ enum UnstableOptions {}
 /// Main processing function. Allows us to return a `Result` so that `main` can print pretty error
 /// messages.
 fn exec(args: UpgradeArgs) -> CargoResult<()> {
-    if !args.offline {
-        let url = registry_url(&find(args.manifest_path.as_deref())?, None)?;
-        update_registry_index(&url, false)?;
-    }
+    let offline = false;
+    let mut index = IndexCache::new(CertsSource::Native);
 
-    let metadata = resolve_ws(args.manifest_path.as_deref(), args.locked, args.offline)?;
+    let metadata = resolve_ws(args.manifest_path.as_deref(), args.locked, offline)?;
     let root_manifest_path = metadata.workspace_root.as_std_path().join("Cargo.toml");
     let manifests = find_ws_members(&metadata);
     let mut manifests = manifests
@@ -212,7 +206,6 @@ fn exec(args: UpgradeArgs) -> CargoResult<()> {
         .collect::<CargoResult<IndexMap<_, Option<_>>>>()?;
     let mut processed_keys = BTreeSet::new();
 
-    let mut updated_registries = BTreeSet::new();
     let mut modified_crates = BTreeSet::new();
     let mut git_crates = BTreeSet::new();
     let mut pinned_present = false;
@@ -295,26 +288,16 @@ fn exec(args: UpgradeArgs) -> CargoResult<()> {
                 {
                     // Update indices for any alternative registries, unless
                     // we're offline.
-                    let registry_url = dependency
-                        .registry()
-                        .map(|registry| registry_url(&manifest_path, Some(registry)))
-                        .transpose()?;
-                    if !args.offline {
-                        if let Some(registry_url) = &registry_url {
-                            if updated_registries.insert(registry_url.to_owned()) {
-                                update_registry_index(registry_url, false)?;
-                            }
-                        }
-                    }
-                    let latest_compatible = semver::VersionReq::parse(&old_version_req)
+                    let registry_url = registry_url(&manifest_path, dependency.registry())?;
+                    let index = index.index(&registry_url)?;
+                    let latest_compatible = VersionReq::parse(&old_version_req)
                         .ok()
                         .and_then(|old_version_req| {
                             get_compatible_dependency(
                                 &dependency.name,
                                 &old_version_req,
                                 rust_version,
-                                &manifest_path,
-                                registry_url.as_ref(),
+                                index,
                             )
                             .ok()
                         })
@@ -324,19 +307,14 @@ fn exec(args: UpgradeArgs) -> CargoResult<()> {
                                 .to_owned()
                         });
                     let is_prerelease = old_version_req.contains('-');
-                    let latest_version = get_latest_dependency(
-                        &dependency.name,
-                        is_prerelease,
-                        rust_version,
-                        &manifest_path,
-                        registry_url.as_ref(),
-                    )
-                    .map(|d| {
-                        d.version()
-                            .expect("registry packages always have a version")
-                            .to_owned()
-                    })
-                    .ok();
+                    let latest_version =
+                        get_latest_dependency(&dependency.name, is_prerelease, rust_version, index)
+                            .map(|d| {
+                                d.version()
+                                    .expect("registry packages always have a version")
+                                    .to_owned()
+                            })
+                            .ok();
                     let latest_incompatible = if latest_version != latest_compatible {
                         latest_version
                     } else {
@@ -475,14 +453,14 @@ fn exec(args: UpgradeArgs) -> CargoResult<()> {
     } else if args.dry_run {
     } else {
         // Ensure lock file is updated and collect data for `recursive`
-        let metadata = resolve_ws(Some(&root_manifest_path), args.locked, args.offline)?;
+        let metadata = resolve_ws(Some(&root_manifest_path), args.locked, offline)?;
         let mut locked = metadata.packages;
 
         let precise_deps = selected_dependencies
             .iter()
             .filter_map(|(name, req)| {
                 req.as_ref()
-                    .and_then(|req| semver::VersionReq::parse(req).ok())
+                    .and_then(|req| VersionReq::parse(req).ok())
                     .and_then(|req| {
                         let precise = precise_version(&req)?;
                         Some((name, (req, precise)))
@@ -494,12 +472,11 @@ fn exec(args: UpgradeArgs) -> CargoResult<()> {
             //
             // Reusing updates (resolve_ws) so we know what lock_version to reference
             for (name, (req, precise)) in &precise_deps {
-                #[allow(clippy::unnecessary_lazy_evaluations)] // requires 1.62
                 for lock_version in locked
                     .iter()
                     .filter(|p| p.name == **name)
                     .map(|p| &p.version)
-                    .filter_map(|v| req.matches(v).then(|| v))
+                    .filter(|v| req.matches(v))
                 {
                     let mut cmd = std::process::Command::new("cargo");
                     cmd.arg("update");
@@ -710,12 +687,7 @@ fn precise_version(version_req: &VersionReq) -> Option<String> {
             matches!(
                 c.op,
                 // Only ops we can determine a precise version from
-                semver::Op::Exact
-                    | semver::Op::GreaterEq
-                    | semver::Op::LessEq
-                    | semver::Op::Tilde
-                    | semver::Op::Caret
-                    | semver::Op::Wildcard
+                Op::Exact | Op::GreaterEq | Op::LessEq | Op::Tilde | Op::Caret | Op::Wildcard
             )
         })
         .filter_map(|c| {
@@ -795,7 +767,7 @@ impl Dep {
             .and_then(|v| semver::Version::parse(v).ok())
         {
             if let Some(new_version_req) = &self.new_version_req {
-                if let Ok(new_version_req) = semver::VersionReq::parse(new_version_req) {
+                if let Ok(new_version_req) = VersionReq::parse(new_version_req) {
                     if !new_version_req.matches(&latest_version) {
                         spec.set_fg(Some(Color::Red));
                     }
@@ -854,7 +826,7 @@ impl Dep {
             .and_then(|v| semver::Version::parse(v).ok())
         {
             if let Some(old_version_req) = &self.old_version_req {
-                if let Ok(old_version_req) = semver::VersionReq::parse(old_version_req) {
+                if let Ok(old_version_req) = VersionReq::parse(old_version_req) {
                     return old_version_req.matches(&latest_version);
                 }
             }
