@@ -6,10 +6,11 @@ use self::Normalization::*;
 use crate::directory::Directory;
 use crate::run::PathDependency;
 use std::cmp;
+use std::mem;
 use std::path::Path;
 
 #[derive(Copy, Clone)]
-pub struct Context<'a> {
+pub(crate) struct Context<'a> {
     pub krate: &'a str,
     pub source_dir: &'a Directory,
     pub workspace: &'a Directory,
@@ -58,6 +59,8 @@ normalizations! {
     AndOthers,
     StripLongTypeNameFiles,
     UnindentAfterHelp,
+    AndOthersVerbose,
+    UnindentMultilineNote,
     // New normalization steps are to be inserted here at the end so that any
     // snapshots saved before your normalization change remain passing.
 }
@@ -73,7 +76,7 @@ normalizations! {
 ///
 /// There is one "preferred" variation which is what we print when the stderr
 /// file is absent or not a match.
-pub fn diagnostics(output: &str, context: Context) -> Variations {
+pub(crate) fn diagnostics(output: &str, context: Context) -> Variations {
     let output = output.replace("\r\n", "\n");
 
     let mut result = Variations::default();
@@ -84,7 +87,7 @@ pub fn diagnostics(output: &str, context: Context) -> Variations {
     result
 }
 
-pub struct Variations {
+pub(crate) struct Variations {
     variations: [String; Normalization::ALL.len()],
 }
 
@@ -107,7 +110,7 @@ impl Variations {
     }
 }
 
-pub fn trim<S: AsRef<[u8]>>(output: S) -> String {
+pub(crate) fn trim<S: AsRef<[u8]>>(output: S) -> String {
     let bytes = output.as_ref();
     let mut normalized = String::from_utf8_lossy(bytes).into_owned();
 
@@ -130,6 +133,7 @@ fn apply(original: &str, normalization: Normalization, context: Context) -> Stri
         normalization,
         context,
         hide_numbers: 0,
+        other_types: None,
     };
     for i in 0..lines.len() {
         if let Some(line) = filter.apply(i) {
@@ -150,6 +154,7 @@ struct Filter<'a> {
     normalization: Normalization,
     context: Context<'a>,
     hide_numbers: usize,
+    other_types: Option<usize>,
 }
 
 impl<'a> Filter<'a> {
@@ -283,7 +288,7 @@ impl<'a> Filter<'a> {
                 } else if line[indent + 4..].starts_with("/rustc/")
                     && line
                         .get(indent + 11..indent + 51)
-                        .map_or(false, is_ascii_lowercase_hex)
+                        .is_some_and(is_ascii_lowercase_hex)
                     && line[indent + 51..].starts_with("/library/")
                 {
                     // --> /rustc/c5c7d2b37780dac1092e75f12ab97dd56c30861e/library/std/src/net/ip.rs:83:1
@@ -301,7 +306,7 @@ impl<'a> Filter<'a> {
                     let hash_end = hash_start + 16;
                     if line
                         .get(hash_start..hash_end)
-                        .map_or(false, is_ascii_lowercase_hex)
+                        .is_some_and(is_ascii_lowercase_hex)
                         && line[hash_end..].starts_with('/')
                     {
                         // --> /home/.cargo/registry/src/github.com-1ecc6299db9ec823/serde_json-1.0.64/src/de.rs:2584:8
@@ -401,11 +406,39 @@ impl<'a> Filter<'a> {
             let trimmed_line = trimmed_line
                 .strip_prefix("= note: ")
                 .unwrap_or(trimmed_line);
-            if trimmed_line.starts_with("the full type name has been written to") {
+            if trimmed_line.starts_with("the full type name has been written to")
+                || trimmed_line.starts_with("the full name for the type has been written to")
+            {
                 return None;
             }
         }
 
+        if self.normalization >= AndOthersVerbose {
+            let trim_start = line.trim_start();
+            if trim_start.starts_with("= help: the following types implement trait ")
+                || trim_start.starts_with("= help: the following other types implement trait ")
+            {
+                self.other_types = Some(0);
+            } else if let Some(count_other_types) = &mut self.other_types {
+                if indent >= 12 && trim_start != "and $N others" {
+                    *count_other_types += 1;
+                    if *count_other_types == 9 {
+                        if let Some(next) = self.all_lines.get(index + 1) {
+                            let next_trim_start = next.trim_start();
+                            let next_indent = next.len() - next_trim_start.len();
+                            if indent == next_indent {
+                                line.replace_range(indent - 2.., "and $N others");
+                            }
+                        }
+                    } else if *count_other_types > 9 {
+                        return None;
+                    }
+                } else {
+                    self.other_types = None;
+                }
+            }
+        }
+
         line = line.replace(self.context.krate, "$CRATE");
         line = replace_case_insensitive(&line, &self.context.source_dir.to_string_lossy(), "$DIR/");
         line = replace_case_insensitive(
@@ -511,22 +544,24 @@ fn unindent(diag: String, normalization: Normalization) -> String {
         normalized.push_str(line);
         normalized.push('\n');
 
-        if indented_line_kind(line, normalization) != IndentedLineKind::Heading {
+        if indented_line_kind(line, &mut false, normalization) != IndentedLineKind::Heading {
             continue;
         }
 
         let mut ahead = lines.clone();
-        let next_line = match ahead.next() {
-            Some(line) => line,
-            None => continue,
+        let Some(next_line) = ahead.next() else {
+            continue;
         };
 
-        if let IndentedLineKind::Code(indent) = indented_line_kind(next_line, normalization) {
+        if let IndentedLineKind::Code(indent) =
+            indented_line_kind(next_line, &mut false, normalization)
+        {
             if next_line[indent + 1..].starts_with("--> ") {
                 let mut lines_in_block = 1;
                 let mut least_indent = indent;
+                let mut previous_line_is_note = false;
                 while let Some(line) = ahead.next() {
-                    match indented_line_kind(line, normalization) {
+                    match indented_line_kind(line, &mut previous_line_is_note, normalization) {
                         IndentedLineKind::Heading => break,
                         IndentedLineKind::Code(indent) => {
                             lines_in_block += 1;
@@ -542,10 +577,11 @@ fn unindent(diag: String, normalization: Normalization) -> String {
                         }
                     }
                 }
+                previous_line_is_note = false;
                 for _ in 0..lines_in_block {
                     let line = lines.next().unwrap();
                     if let IndentedLineKind::Code(_) | IndentedLineKind::Other(_) =
-                        indented_line_kind(line, normalization)
+                        indented_line_kind(line, &mut previous_line_is_note, normalization)
                     {
                         let space = line.find(' ').unwrap();
                         normalized.push_str(&line[..space]);
@@ -562,7 +598,13 @@ fn unindent(diag: String, normalization: Normalization) -> String {
     normalized
 }
 
-fn indented_line_kind(line: &str, normalization: Normalization) -> IndentedLineKind {
+fn indented_line_kind(
+    line: &str,
+    previous_line_is_note: &mut bool,
+    normalization: Normalization,
+) -> IndentedLineKind {
+    let previous_line_was_note = mem::replace(previous_line_is_note, false);
+
     if let Some(heading_len) = if line.starts_with("error") {
         Some("error".len())
     } else if line.starts_with("warning") {
@@ -578,7 +620,11 @@ fn indented_line_kind(line: &str, normalization: Normalization) -> IndentedLineK
     if line.starts_with("note:")
         || line == "..."
         || normalization >= UnindentAfterHelp && line.starts_with("help:")
+        || normalization >= UnindentMultilineNote
+            && previous_line_was_note
+            && line.starts_with("      ")
     {
+        *previous_line_is_note = true;
         return IndentedLineKind::Note;
     }
 
